__name__ = 'aiogoogle'
__about__ = "An async-framework agnostic async library for Google's Discovery Service"
__url__ = 'https://github.com/omarryhan/aiogoogle'
__version_info__ = ('0', '0', '1b1')
__version__ = '.'.join(__version_info__)
__author__ = 'Omar Ryhan'
__author_email__ = 'omarryhan@gmail.com'
__maintainer__ = 'Omar Ryhan'
__license__ = 'GNU'
__copyright__ = '(c) 2018 by Omar Ryhan'