# TODO: Test requesr add query param
# TODO: Test request rm query param
# TODO: Test Request.from response
# TODO: Test next page generator
# TODO: Test response.__call__ returns pagination gen
# TODO: Test next_page returns a valid request